package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.representation.Form;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.jersey.oauth.client.OAuthClientFilter;
import com.sun.jersey.oauth.signature.HMAC_SHA1;
import com.sun.jersey.oauth.signature.OAuthParameters;
import com.sun.jersey.oauth.signature.OAuthSecrets;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

/**
 *
 * @author joeri
 */
public class GoogleExternalNetwork extends ExternalNetwork {
  public GoogleExternalNetwork(String consumerKey, String consumerSecret) {
    super(consumerKey, consumerSecret, "google");
  }

  public GoogleExternalNetwork(String consumerKey, String consumerSecret, String name) {
    super(consumerKey, consumerSecret, name);
  }

  @Override
  public Response connect(String callback) {
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0");
    OAuthSecrets secrets = new OAuthSecrets().consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
    WebResource wr = client.resource("https://www.google.com");
    wr.addFilter(filter);

    MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
    queryParams.add("scope", "https://www-opensocial.googleusercontent.com/api/people/");

    ClientResponse clientResponse = wr.path("accounts/OAuthGetRequestToken").queryParams(queryParams).get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      Form responseParameters = clientResponse.getEntity(Form.class);

      String token = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
      String tokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

      ExternalToken externalToken = new ExternalToken();
      externalToken.setNetwork(getName());
      externalToken.setType(ExternalToken.Type.REQUEST_TOKEN);
      externalToken.setCreationDate(System.currentTimeMillis());
      externalToken.setToken(token);
      externalToken.setSecret(tokenSecret);
      externalNetworkBean.createExternalToken(externalToken);

      try {
        return Response.seeOther(new URI("https://www.google.com/accounts/OAuthAuthorizeToken?" + OAuthParameters.TOKEN + "=" + URLEncoder.encode(token, "UTF-8") + "&" + OAuthParameters.CALLBACK + "=" + URLEncoder.encode(callback, "UTF-8"))).build();
      } catch (UnsupportedEncodingException ex) {
        throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
      } catch (URISyntaxException ex) {
        throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
      }
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

  @Override
  public ExternalToken callback(String requestToken, String verifier) {
    ExternalToken externalRequestToken = externalNetworkBean.findExternalToken(getName(), ExternalToken.Type.REQUEST_TOKEN, requestToken);
    if (externalRequestToken != null) {
      WebResource wr = client.resource("https://www.google.com");
      OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
              consumerKey(consumerKey).version("1.0").token(externalRequestToken.getToken()).verifier(verifier);
      OAuthSecrets secrets = new OAuthSecrets().tokenSecret(externalRequestToken.getSecret()).consumerSecret(consumerSecret);
      OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
      wr.addFilter(filter);

      ClientResponse clientResponse = wr.path("accounts/OAuthGetAccessToken").get(ClientResponse.class);
      if (clientResponse.getStatus() == 200) {
        Form responseParameters = clientResponse.getEntity(Form.class);

        String accessToken = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
        String accessTokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

        // delete request token
        externalNetworkBean.removeExternalToken(externalRequestToken.getId());

        ExternalToken externalAccessToken = new ExternalToken();
        externalAccessToken.setNetwork(getName());
        externalAccessToken.setType(ExternalToken.Type.ACCESS_TOKEN);
        externalAccessToken.setCreationDate(System.currentTimeMillis());
        externalAccessToken.setToken(accessToken);
        externalAccessToken.setSecret(accessTokenSecret);
        externalAccessToken = externalNetworkBean.createExternalToken(externalAccessToken);

        return externalAccessToken;
      } else {
        throw new WebApplicationException(Response.Status.BAD_REQUEST);
      }
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

  @Override
  public OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException {
    WebResource wr = client.resource("https://www-opensocial.googleusercontent.com");
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").token(accessToken.getToken());
    OAuthSecrets secrets = new OAuthSecrets().tokenSecret(accessToken.getSecret()).consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
    wr.addFilter(filter);

    ClientResponse clientResponse = wr.path("/api/people/@me/@self").accept(MediaType.APPLICATION_XML_TYPE).get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      String json = clientResponse.getEntity(String.class);

      String id = "";
      String screenName = "";
      String firstName = "";
      String lastName = "";
      String pictureUrl = "";
      try {
        JSONObject jsonObject = new JSONObject(json);

        id = jsonObject.getJSONObject("entry").getString("id");
        screenName = jsonObject.getJSONObject("entry").getString("displayName");
        firstName = jsonObject.getJSONObject("entry").getJSONObject("name").getString("givenName");
        lastName = jsonObject.getJSONObject("entry").getJSONObject("name").getString("familyName");
        pictureUrl = jsonObject.getJSONObject("entry").getString("thumbnailUrl");
      } catch (JSONException ex) {
        Constants.LOGGER.log(Level.SEVERE, null, ex);
      }

      OnlineAccount onlineAccount = externalNetworkBean.findOnlineAccount(appKey, getName(), id);
      if (onlineAccount == null) {
        onlineAccount = new OnlineAccount();
				onlineAccount.setAppKey(appKey);
        onlineAccount.setCreationDate(System.currentTimeMillis());
        onlineAccount.setNetwork(getName());
        onlineAccount.setAccountId(id);
      }

      if (onlineAccount.getUser() == null) {
        User user = new User();
        user.setScreenName(screenName);
        user.setFirstName(firstName);
        user.setLastName(lastName);
        user.setDepiction(pictureUrl);
        onlineAccount.setUser(user);
      }

      return onlineAccount;
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

	@Override
	public List<Stream> getStream(OnlineAccount account) {
		return new LinkedList<Stream>();
	}

	@Override
	public List<Friend> getFriends(OnlineAccount account) {
		return new LinkedList<Friend>();
	}
}
