package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.entity.Actor;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.filter.LoggingFilter;
import com.sun.jersey.api.representation.Form;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.jersey.oauth.client.OAuthClientFilter;
import com.sun.jersey.oauth.signature.HMAC_SHA1;
import com.sun.jersey.oauth.signature.OAuthParameters;
import com.sun.jersey.oauth.signature.OAuthSecrets;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.UUID;
import java.util.logging.Level;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

/**
 *
 * @author joeri
 */
public class TwitterExternalNetwork extends ExternalNetwork {
	private static DateFormat dfStatus = new SimpleDateFormat("EE MMM dd HH:mm:ss '+0000' yyyy", Locale.US);
  
  public TwitterExternalNetwork(String consumerKey, String consumerSecret) {
    super(consumerKey, consumerSecret, "twitter");
  }

  public TwitterExternalNetwork(String consumerKey, String consumerSecret, String name) {
    super(consumerKey, consumerSecret, name);
  }

  @Override
  public Response connect(String callback) {
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0");
    OAuthSecrets secrets = new OAuthSecrets().consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
    WebResource wr = client.resource("https://api.twitter.com");
    wr.addFilter(filter);

    ClientResponse clientResponse = wr.path("oauth/request_token").post(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      Form responseParameters = clientResponse.getEntity(Form.class);

      String token = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
      String tokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

      ExternalToken externalToken = new ExternalToken();
      externalToken.setNetwork(getName());
      externalToken.setType(ExternalToken.Type.REQUEST_TOKEN);
      externalToken.setCreationDate(System.currentTimeMillis());
      externalToken.setToken(token);
      externalToken.setSecret(tokenSecret);
      externalNetworkBean.createExternalToken(externalToken);

      try {
        return Response.seeOther(new URI("https://api.twitter.com/oauth/authenticate?" + OAuthParameters.TOKEN + "=" + URLEncoder.encode(token, "UTF-8") + "&" + OAuthParameters.CALLBACK + "=" + URLEncoder.encode(callback, "UTF-8"))).build();
      } catch (UnsupportedEncodingException ex) {
        throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
      } catch (URISyntaxException ex) {
        throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
      }
    } else {
      Constants.LOGGER.log(Level.INFO, "Twitter did not allow us to connect, statuscode = {0}", clientResponse.getStatus());
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

  @Override
  public ExternalToken callback(String requestToken, String verifier) {
    ExternalToken externalRequestToken = externalNetworkBean.findExternalToken(getName(), ExternalToken.Type.REQUEST_TOKEN, requestToken);
    if (externalRequestToken != null) {
      WebResource wr = client.resource("https://api.twitter.com");
      OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
              consumerKey(consumerKey).version("1.0").token(externalRequestToken.getToken()).verifier(verifier);
      OAuthSecrets secrets = new OAuthSecrets().tokenSecret(externalRequestToken.getSecret()).consumerSecret(consumerSecret);
      OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
      wr.addFilter(filter);

      ClientResponse clientResponse = wr.path("oauth/access_token").get(ClientResponse.class);
      if (clientResponse.getStatus() == 200) {
        Form responseParameters = clientResponse.getEntity(Form.class);

        String accessToken = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
        String accessTokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

        // delete request token
        externalNetworkBean.removeExternalToken(externalRequestToken.getId());

        ExternalToken externalAccessToken = new ExternalToken();
        externalAccessToken.setNetwork(getName());
        externalAccessToken.setType(ExternalToken.Type.ACCESS_TOKEN);
        externalAccessToken.setCreationDate(System.currentTimeMillis());
        externalAccessToken.setToken(accessToken);
        externalAccessToken.setSecret(accessTokenSecret);
        externalAccessToken = externalNetworkBean.createExternalToken(externalAccessToken);

        return externalAccessToken;
      } else {
        throw new WebApplicationException(Response.Status.BAD_REQUEST);
      }
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

  @Override
  public OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException {
    WebResource wr = client.resource("https://api.twitter.com");
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").token(accessToken.getToken());
    OAuthSecrets secrets = new OAuthSecrets().tokenSecret(accessToken.getSecret()).consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
    wr.addFilter(filter);

    ClientResponse clientResponse = wr.path("1/account/verify_credentials.json").get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      String credentials = clientResponse.getEntity(String.class);

			String id = "";
      String screenName = "";
      String userName = "";
      String pictureUrl = "";

			try {
				JSONObject jsonCredentials = new JSONObject(credentials);
				
				id = jsonCredentials.getString("id");
				screenName = jsonCredentials.getString("screen_name");
				userName = jsonCredentials.getString("name");
				pictureUrl = jsonCredentials.getString("profile_image_url");
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

      OnlineAccount onlineAccount = externalNetworkBean.findOnlineAccount(appKey, getName(), id);
      if (onlineAccount == null) {
        onlineAccount = new OnlineAccount();
				onlineAccount.setAppKey(appKey);
        onlineAccount.setCreationDate(System.currentTimeMillis());
        onlineAccount.setNetwork(getName());
        onlineAccount.setAccountId(id);
      }

      if (onlineAccount.getUser() == null) {
        User user = new User();
        user.setScreenName(screenName);
        user.setFirstName(userName);
        user.setDepiction(pictureUrl);
        onlineAccount.setUser(user);
      }

      return onlineAccount;
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

	@Override
	public List<Stream> getStream(OnlineAccount account) {
    WebResource wr = client.resource("https://api.twitter.com");
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
    OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

		wr.addFilter(new LoggingFilter());
    wr.addFilter(filter);
    ClientResponse clientResponse = wr.path("1/statuses/home_timeline.json")
						.queryParam("include_rts", "false")
						.get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      String statuses = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "Twitter statuses: {0}", statuses);

			List<Stream> streams = new LinkedList<Stream>();

			try {
				JSONArray jsonStatuses = new JSONArray(statuses);
				for (int i = 0; i < jsonStatuses.length(); i++) {
					JSONObject jsonStatus = jsonStatuses.getJSONObject(i);
					Stream stream = new Stream();
					stream.setNetwork(getName());
					stream.setUid(UUID.randomUUID().toString());
					stream.setRemoteId(jsonStatus.getString("id"));
					stream.setContent(jsonStatus.getString("text"));

					try {
						String createdAt = jsonStatus.getString("created_at");
						stream.setCreationDate(dfStatus.parse(createdAt).getTime());
					} catch (ParseException ex) {
						Constants.LOGGER.log(Level.WARNING, null, ex);
						stream.setCreationDate(System.currentTimeMillis());
					}

					JSONObject user = jsonStatus.optJSONObject("user");
					if (user != null) {
						Actor actor = new Actor();
						actor.setUid(UUID.randomUUID().toString());
						actor.setRemoteId(String.valueOf(user.optLong("id")));
						actor.setScreenName(user.optString("name"));
						actor.setDepiction(user.optString("profile_image_url"));
						stream.setAuthor(actor);
					}

					streams.add(stream);
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

			return streams;
    } else {
			Constants.LOGGER.log(Level.FINER, "Twitter response not 200: {0}", clientResponse.getEntity(String.class));
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
	}

	@Override
	public List<Friend> getFriends(OnlineAccount account) {
    WebResource wr = client.resource("https://api.twitter.com");
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
    OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

		wr.addFilter(new LoggingFilter());
    wr.addFilter(filter);
    ClientResponse clientResponse = wr.path("1/friends/ids.json")
						.queryParam("cursor", "-1")
						.queryParam("user_id", account.getAccountId())
						.get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      String twitterFriends = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "Twitter friendIds: {0}", twitterFriends);

			List<Friend> friends = new LinkedList<Friend>();

			try {
				JSONObject jsonFriends = new JSONObject(twitterFriends);
				JSONArray jsonFriendIds = jsonFriends.optJSONArray("ids");
				List<Long> friendIds = new LinkedList<Long>();
				for (int i = 0; i < jsonFriendIds.length(); i++) {
					friendIds.add(jsonFriendIds.getLong(i));
					if (friendIds.size() == 100) {
						friends.addAll(friendsById(friendIds, account));
						friendIds.clear();
					}
				}

				if (friendIds.size() > 0) {
					friends.addAll(friendsById(friendIds, account));
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

			return friends;
    } else {
			Constants.LOGGER.log(Level.FINER, "Twitter response not 200: {0}", clientResponse.getEntity(String.class));
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
	}

	private List<Friend> friendsById(List<Long> friendIds, OnlineAccount account) {
    WebResource wr = client.resource("https://api.twitter.com");
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
    OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

		StringBuilder sbFriendIds = new StringBuilder();
		for (Long friendId : friendIds) {
			if (sbFriendIds.length() > 0) sbFriendIds.append(",");
			sbFriendIds.append(friendId);
		}

		MultivaluedMap<String, String> postParams = new MultivaluedMapImpl();
		postParams.add("user_id", sbFriendIds.toString());

		wr.addFilter(new LoggingFilter());
    wr.addFilter(filter);
    ClientResponse clientResponse = wr.path("1/users/lookup.json")
						.post(ClientResponse.class, postParams);
    if (clientResponse.getStatus() == 200) {
      String twitterFriends = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "Twitter friends: {0}", twitterFriends);

			List<Friend> friends = new LinkedList<Friend>();

			try {
				JSONArray jsonFriends = new JSONArray(twitterFriends);
				for (int i = 0; i < jsonFriends.length(); i++) {
					JSONObject jsonFriend = jsonFriends.getJSONObject(i);
					Friend friend = new Friend();
					friend.setNetwork(getName());
					friend.setUid(UUID.randomUUID().toString());
					friend.setRemoteId(String.valueOf(jsonFriend.optLong("id")));
					friend.setScreenName(jsonFriend.optString("screen_name"));
					friend.setFirstName(jsonFriend.optString("name"));
					friend.setDepiction(jsonFriend.optString("profile_image_url"));
					friends.add(friend);
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

			return friends;
    } else {
			Constants.LOGGER.log(Level.FINER, "Twitter response not 200: {0}", clientResponse.getEntity(String.class));
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
	}
}
