package com.lodgon.dali.core.social.ejb;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.ejb.UserBean;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.Constants;
import com.lodgon.dali.core.social.DaliCoreExternalNetwork;
import com.lodgon.dali.core.social.ExternalNetwork;
import com.lodgon.dali.core.social.ExternalNetworkLocator;
import com.lodgon.dali.core.social.FacebookExternalNetwork;
import com.lodgon.dali.core.social.GoogleExternalNetwork;
import com.lodgon.dali.core.social.GooglePlusExternalNetwork;
import com.lodgon.dali.core.social.LinkedinExternalNetwork;
import com.lodgon.dali.core.social.TwitterExternalNetwork;
import com.lodgon.dali.core.social.entity.Actor;
import com.lodgon.dali.core.social.entity.Company;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.Job;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.Level;
import javax.ejb.Stateless;
import javax.enterprise.inject.Any;
import javax.enterprise.inject.Instance;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

/**
 *
 * @author joeri
 */
@Stateless
public class DaliCoreSocialBean {

	@Inject
	@Any
	private Instance<ExternalNetworkLocator> externalNetworkLocators;
	@PersistenceContext
	EntityManager em;
	@Inject
	UserBean userBean;
	
	//TODO: temporary implemented with static hashmap. Add expiration time.
	private static final Map<String, Long> verifiers = new HashMap();

	public ExternalToken createExternalToken(ExternalToken externalToken) {
		em.persist(externalToken);
		return externalToken;
	}

	public ExternalToken findExternalToken(String network, ExternalToken.Type type, String token) {
		Query q = em.createNamedQuery("ExternalToken.findByNetworkAndTypeAndToken");
		q.setParameter("network", network);
		q.setParameter("type", type);
		q.setParameter("token", token);
		List<ExternalToken> tokens = q.getResultList();
		if (tokens.isEmpty()) {
			return null;
		} else {
			return tokens.iterator().next();
		}
	}

	public void removeExternalToken(Long externalTokenId) {
		ExternalToken externalToken = em.find(ExternalToken.class, externalTokenId);
		if (externalToken != null) {
			em.remove(externalToken);
		}
	}

	public OnlineAccount createOnlineAccount(OnlineAccount onlineAccount) {
		em.persist(onlineAccount);
		return onlineAccount;
	}

	public OnlineAccount updateOnlineAccount(OnlineAccount onlineAccount) {
		return em.merge(onlineAccount);
	}

	public OnlineAccount findOnlineAccount(Long onlineAccountId) {
		return em.find(OnlineAccount.class, onlineAccountId);
	}

	public OnlineAccount findOnlineAccount(String appKey, String network, String accountId) {
		Query q = em.createNamedQuery("OnlineAccount.findByAppKeyAndNetworkAndAccountId");
		q.setParameter("appKey", appKey);
		q.setParameter("network", network);
		q.setParameter("accountId", accountId);
		List<OnlineAccount> accounts = q.getResultList();
		if (accounts.isEmpty()) {
			return null;
		} else {
			return accounts.iterator().next();
		}
	}

	public List<OnlineAccount> findOnlineAccountsByUser(String appKey, Integer userId) {
		Query q = em.createNamedQuery("OnlineAccount.findByAppKeyAndUser");
		q.setParameter("appKey", appKey);
		q.setParameter("user", em.find(User.class, userId));
		return q.getResultList();
	}

	public Collection<Stream> getStream(String appKey, String uid, Map<String, String> consumerKeys) throws DaliCoreException {
		Set<Stream> streams = new TreeSet<Stream>(new Comparator<Stream>() {
			@Override
			public int compare(Stream s1, Stream s2) {
				return s1.getCreationDate() == s2.getCreationDate() ? s1.getUid().compareTo(s2.getUid()) * -1 : (s1.getCreationDate() < s2.getCreationDate() ? 1 : -1);
			}
		});
		User user = userBean.getByUid(appKey, uid);
		List<OnlineAccount> accounts = findOnlineAccountsByUser(appKey, user.getId());
		Constants.LOGGER.log(Level.FINE, "User with uid {0} has the following online accounts: {1}", new Object[]{user.getUid(), accounts});
		for (OnlineAccount account : accounts) {
			String network = account.getNetwork();
			ExternalNetwork externalNetwork = getExternalNetwork(appKey, network, consumerKeys);
			try {
				streams.addAll(externalNetwork.getStream(account));
			} catch (Exception ex) {
				Constants.LOGGER.log(Level.SEVERE, "Something went wrong while requesting stream from " + externalNetwork.getName(), ex);
			}
		}
		Constants.LOGGER.log(Level.FINER, "returning stream: {0}", streams);
		return streams;
	}

  public Collection<Company> searchCompanies(String appKey, String uid, String searchKey, Map<String, String> consumerKeys) throws DaliCoreException {
    Set<Company> answer = new HashSet();
    User user = userBean.getByUid(appKey, uid);
    List<OnlineAccount> accounts = findOnlineAccountsByUser(appKey, user.getId());
    Constants.LOGGER.log(Level.FINE, "User with uid {0} has the following online accounts: {1}", new Object[]{user.getUid(), accounts});
    for (OnlineAccount account : accounts) {
      String network = account.getNetwork();
      ExternalNetwork externalNetwork = getExternalNetwork(appKey, network, consumerKeys);
      try {
        answer.addAll(externalNetwork.searchCompanies(account, searchKey));
      } catch (Exception ex) {
        Constants.LOGGER.log(Level.SEVERE, "Something went wrong while requesting companies from " + externalNetwork.getName(), ex);
      }
    }
    return answer;
  }
  
  public Company getCompany(String appKey, String uid, String companyId, String network, Map<String, String> consumerKeys) throws DaliCoreException {
    User user = userBean.getByUid(appKey, uid);
    List<OnlineAccount> accounts = findOnlineAccountsByUser(appKey, user.getId());
    for (OnlineAccount account : accounts) {
      String candidate = account.getNetwork();
      if (candidate.equals(network)) {
        ExternalNetwork externalNetwork = getExternalNetwork(appKey, network, consumerKeys);
        Company company = externalNetwork.getCompany(account, companyId);
        return company;
      }
    }
    Constants.LOGGER.log(Level.SEVERE, "Can't find a company with id {0} on network {1}", new Object[]{companyId, network});
    return null;
  }
  
  public Collection<Job> searchJobs (String appKey, String uid, String companyName, String network,  Map<String, String> consumerKeys) throws DaliCoreException {
    User user = userBean.getByUid(appKey, uid);
    List<OnlineAccount> accounts = findOnlineAccountsByUser(appKey, user.getId());
    for (OnlineAccount account : accounts) {
      String candidate = account.getNetwork();
      if (candidate.equals(network)) {
        ExternalNetwork externalNetwork = getExternalNetwork(appKey, network, consumerKeys);
        Collection<Job> answer = externalNetwork.searchJobsByCompany(account, companyName);
        return answer;
      }
    }
    Constants.LOGGER.log(Level.SEVERE, "Can't find jobs on network {0} since it is not in the onlineAccounts", network);
    return null;
  }

  public Collection<Actor> searchPeople (String appKey, String uid, String keyword, Map<String, String> consumerKeys) throws DaliCoreException {
    Collection<Actor> answer = new HashSet<Actor>();
    User user = userBean.getByUid(appKey, uid);
		List<OnlineAccount> accounts = findOnlineAccountsByUser(appKey, user.getId());
		Constants.LOGGER.log(Level.FINE, "User with uid {0} has the following online accounts: {1}", new Object[]{user.getUid(), accounts});
		for (OnlineAccount account : accounts) {
			String network = account.getNetwork();
			ExternalNetwork externalNetwork = getExternalNetwork(appKey, network, consumerKeys);
			try {
				answer.addAll(externalNetwork.searchPeople(account, keyword));
			} catch (Exception ex) {
				Constants.LOGGER.log(Level.SEVERE, "Something went wrong while searching people from " + externalNetwork.getName(), ex);
			}
		}
		Constants.LOGGER.log(Level.FINER, "returning people: {0}", answer);
		return answer;
  }
  
	public Collection<Friend> getFriends(String appKey, String uid, Map<String, String> consumerKeys) throws DaliCoreException {
		Set<Friend> friends = new TreeSet<Friend>(new Comparator<Friend>() {
			@Override
			public int compare(Friend f1, Friend f2) {
				return f1.getCreationDate() == f2.getCreationDate() ? f1.getUid().compareTo(f2.getUid()) * -1 : (f1.getCreationDate() < f2.getCreationDate() ? 1 : -1);
			}
		});
		User user = userBean.getByUid(appKey, uid);
		List<OnlineAccount> accounts = findOnlineAccountsByUser(appKey, user.getId());
		Constants.LOGGER.log(Level.FINE, "User with uid {0} has the following online accounts: {1}", new Object[]{user.getUid(), accounts});
		for (OnlineAccount account : accounts) {
			String network = account.getNetwork();
			ExternalNetwork externalNetwork = getExternalNetwork(appKey, network, consumerKeys);
			try {
				friends.addAll(externalNetwork.getFriends(account));
			} catch (Exception ex) {
				Constants.LOGGER.log(Level.SEVERE, "Something went wrong while requesting friends from " + externalNetwork.getName(), ex);
			}
		}
		Constants.LOGGER.log(Level.FINER, "returning friends: {0}", friends);
		return friends;
	}

	/**
	 * Remove the specific network from the network-list for this user
	 *
	 * @param network the name of the network (e.g. facebook)
	 * @param uid the UUID of the user
	 */
	public void disconnect(String appKey, String network, String uid) throws DaliCoreException {
		Constants.LOGGER.log(Level.FINE, "Disconnect user {0} from network {1} with appkey {2}", new Object[]{uid, network, appKey});
		User user = userBean.getByUid(appKey, uid);
		Query q = em.createNamedQuery("OnlineAccount.findByUserAndNetwork");
		q.setParameter("user", user);
		q.setParameter("network", network);
		List<OnlineAccount> resultList = q.getResultList();
		Constants.LOGGER.log(Level.FINE, "results for user = {0}: {1}", new Object[]{user, resultList});
		if (resultList.size() > 0) {
			for (OnlineAccount act : resultList) {
				em.remove(act);
			}
		}
	}

	public ExternalNetwork getExternalNetwork(String appKey, String network, Map<String, String> consumerKeys) throws WebApplicationException {
		Constants.LOGGER.log(Level.FINE, "Get External network, appkey = {0}, network = {1}", new Object[]{appKey, network});
		if (externalNetworkLocators.isUnsatisfied() || externalNetworkLocators.isAmbiguous()) {
			Constants.LOGGER.fine("unsatisfied or ambiguous external network");
			String consumerKey = consumerKeys.get("dalicore.externalnetwork." + network + ".consumerkey");
			String consumerSecret = consumerKeys.get("dalicore.externalnetwork." + network + ".consumersecret");

			if ("dalicore".equals(network)) {
				return new DaliCoreExternalNetwork(consumerKey, consumerSecret, network);
			} else if ("facebook".equals(network)) {
				return new FacebookExternalNetwork(consumerKey, consumerSecret, network);
			} else if ("google".equals(network)) {
				return new GoogleExternalNetwork(consumerKey, consumerSecret, network);
			} else if ("googleplus".equals(network)) {
				return new GooglePlusExternalNetwork(consumerKey, consumerSecret, network);
			} else if ("linkedin".equals(network)) {
				return new LinkedinExternalNetwork(consumerKey, consumerSecret, network);
			} else if ("twitter".equals(network)) {
				return new TwitterExternalNetwork(consumerKey, consumerSecret, network);
			} else {
				try {
					Class<? extends ExternalNetwork> externalNetworkClass = (Class<? extends ExternalNetwork>) Class.forName(network);
					return externalNetworkClass.getConstructor(String.class, String.class).newInstance(consumerKey, consumerSecret, network);
				} catch (Exception ex) {
					StringWriter sw = new StringWriter();
					PrintWriter pw = new PrintWriter(sw);
					ex.printStackTrace(pw);
					pw.close();
					throw new WebApplicationException(ex, Response.ok(sw.getBuffer().toString()).status(Response.Status.BAD_REQUEST).build());
				}
			}
		} else {
			ExternalNetworkLocator locator = externalNetworkLocators.iterator().next();
			Constants.LOGGER.log(Level.FINE, "unique external network locator = {0}", locator);
			return locator.create(appKey, network);
		}
	}

	public String createVerifier(OnlineAccount account) {
		long verifier = System.nanoTime();
		verifiers.put(account.getUser().getUid(), verifier);
		return String.valueOf(verifier);
	}
	
	public boolean verifyVerifier(String uid, String verifier) {
		//TODO: make it possible to repeat this verification for a certain interval?
		Long check = verifiers.remove(uid);
		boolean verified = false;
		if (check != null) {
			if (String.valueOf(check).equals(verifier)) {
				verified = true;		
			}
		}		
		return verified;
	}
}
